#ifndef _s_Interfaces
#define _s_Interfaces

#include <vector>
#include <math.h>
#include <cstdlib> 
#include <iostream>
using namespace std;



const int GMSG_START_MOVE_FORWARD =  1;
const int GMSG_STOP_MOVE_FORWARD  = -1;
const int GMSG_MOVE_TO_LOC =       200;
const int GMSG_CHANGE_ANGLES =     201;
const int GMSG_SHOOT =             202;
const int GMSG_DEATH =             203;
const int GMSG_ACCELERATE =        204;
const int GMSG_DAMAGE     =        205; 
const int GMSG_FRAG       =        206;
const int GMSG_TELEPORT   =        207;
const int GMSG_POWERUP_PICKED  =   301;
const int GMSG_POWERUP_APPEAR  =   302;
const int GMSG_MUSIC_TOGGLE  =     401;


char* getMsgString(int type);

const int PBPNO   =		     6;
const int PBPT_TRAP =		     0;
const int PBPT_ARMOR =		     1;
const int PBPT_GUN =		     2;
const int PBPT_ENGINE =		     3;
const int PBPT_QUAD =		     4;
const int PBPT_PHASE	 =	     5;

const int RESOLUTION_WIDTH = 1024;
const int RESOLUTION_HIGHT = 768;




struct Vector3f
{
	Vector3f(){}
	Vector3f( float X, float Y, float Z)
	{
		x=X;
		y=Y;
		z=Z;
	}
	Vector3f operator- (Vector3f v)
	{
		Vector3f tmp = *this;
		tmp.x -= v.x;
		tmp.y -= v.y;
		tmp.z -= v.z;
		return tmp;
	}
	Vector3f operator+ (Vector3f v)
	{
		Vector3f tmp = *this;
		tmp.x += v.x;
		tmp.y += v.y;
		tmp.z += v.z;
		return tmp;
	}
	Vector3f operator* (float f)
	{
		Vector3f tmp = *this;
		tmp.x *= f;
		tmp.y *= f;
		tmp.z *= f;
		return tmp;
	}
	
	bool operator== ( Vector3f v)
	{
		if ( absf(x-v.x)+absf(y-v.y)+absf(z-v.z)<0.05 ) return true; else return false;
		//return (x==v.x)&&(y==v.y)&&(z==v.z);
	}
	
	Vector3f CrossProduct(Vector3f x);
	float DotProduct(Vector3f v)
	{
		return x*v.x+y*v.y+z*v.z;
	}
	Vector3f Rescale(float l);
	float Length()
	{
		return sqrt( x*x + y*y + z*z );
	}
	
	float x, y, z;

	private:
	 float absf(float f)
	{
		return (f<0) ? -f : f;
	}

};

struct Vector4f : Vector3f
{

	Vector4f()
	{
	}
	Vector4f(Vector3f v,float w)
	{
		x=v.x;
		y=v.y;
		z=v.z;
		w=w;
	}

	Vector4f(float X, float Y, float Z, float W)
	{
		x= X;
		y=Y;
		z=Z;
		w=W;
	}

	/*Vector4f operator= (Vector4f v)
	{
		x=v.x;
		y=v.y;
		z=v.z;
		w=v.w;
		return v;
	}*/

	float w;
};

struct Line2vf
{
	Line2vf(){};
	Line2vf(Vector3f V1,Vector3f V2)
	{
		v1 = V1;
		v2 = V2;
	}

	bool operator==( Line2vf l)
	{
		if ( ( (v1==l.v1) && (v2==l.v2) ) || ( (v1==l.v2) && (v2==l.v1) ) )
			return 1;
		else
			return 0;
	}

	

	Vector3f v1,v2;
};

struct Edge
{
	Line2vf line;
	int l1, l2;
	int v1[3];
	int v2[3];
};

bool Less(Line2vf p, Line2vf l);
bool LessInv(Line2vf p, Line2vf l);
bool LessCross(Line2vf p, Line2vf l);
struct Vector3d 
{
	Vector3d(){}
	Vector3d( double X, double Y, double Z)
	{
		x=X;
		y=Y;
		z=Z;
	}
	Vector3d operator- (Vector3d v)
	{
		Vector3d tmp = *this;
		tmp.x -= v.x;
		tmp.y -= v.y;
		tmp.z -= v.z;
		return tmp;
	}
	Vector3d operator+ (Vector3d v)
	{
		Vector3d tmp = *this;
		tmp.x += v.x;
		tmp.y += v.y;
		tmp.z += v.z;
		return tmp;
	}
	Vector3d operator* (float f)
	{
		Vector3d tmp = *this;
		tmp.x *= f;
		tmp.y *= f;
		tmp.z *= f;
		return tmp;
	}
	bool operator== ( Vector3d v)
	{
		if ( (x==v.x)&&(y==v.y)&&(z==v.z) ) return true; else return false;
	}
	
	Vector3d CrossProduct(Vector3d v)
	{
		return Vector3d( y*v.z - z*v.y,  z*v.x - x*v.z,  x*v.y - y*v.x );
	}
	double DotProduct(Vector3d v)
	{
		return x*v.x+y*v.y+z*v.z;
	}
	//Vector3f Rescale(double l);
	double Length()
	{
		return sqrt( x*x + y*y + z*z );
	}
	
	Vector3d operator= (Vector3f v)
	{
		x = v.x;
		y = v.y;
		z = v.z;
		return (*this);
	}

	double x, y, z;
};

struct Vector2f
{
	Vector2f(){}
	Vector2f(float U, float V)
		{
			u=U;
			v=V;
		}
	Vector2f operator- (Vector2f v)
	{
		Vector2f tmp = *this;
		tmp.u -= v.u;
		tmp.v -= v.v;
		return tmp;
	}
	Vector2f operator+ (Vector2f v)
	{
		Vector2f tmp = *this;
		tmp.u += v.u;
		tmp.v += v.v;
		return tmp;
	}
	float u,v;
	
};

struct SGraphicObject
{
	
	unsigned int* pIndexArray;
	unsigned int Texture;
	unsigned int iShader;
	long iIndexCount;
	float x,y,z;
	float color[4];
	float diff;
	float spec;
	unsigned int iIndexBufferId;
	vector<Edge> edges; 
	unsigned int iEdgeCount;
 	/*~SGraphicObject()
	{
	 delete pIndexArray;
	}*/
	
};

struct SLightObject
{
	float x,y,z;
	float ax;
    float,ay;
//	float near;
//    float far
    float width;
    float height;
	unsigned int resWidth, resHeight;
};


struct SGameMsg
{
	int type;
	int size;
	int target;
	int executed;
	int late;
	float* data; 
	unsigned long TimeStemp;
	bool Voided;
	vector <SGameMsg *> childeren;

	SGameMsg()
	{
	   late=executed = false;
	    data = NULL;
	   Voided = false;
	}
	/*~SGameMsg()
	{
	  if (data != NULL)
	  delete data;
	}*/
};

struct SGameObject
{

	SGameObject()
		{
			size = 0;
			//iActive = 1;
		}

	int type;
	int iGraphicObjectId;
	unsigned int  iTexture1Id, iTexture2Id;
	Vector3f pos;
	Vector3f rot;
	float front, back, left, right; 
	float color[4]; 
	int size;
	float* data;
	int iActive;
	unsigned long iTimer;
	virtual SGameMsg* Action(unsigned long t, int i){return NULL;}
};


struct SGamePowerup : SGameObject
{
	SGamePowerup(){}
	SGamePowerup(SGameObject o)
	{
		type = o.type;
		iGraphicObjectId = o.iGraphicObjectId;
		iTexture1Id = o.iTexture1Id;
		iTexture2Id = o.iTexture2Id;
		pos = o.pos;
		rot = o.rot;
		color[0] = o.color[0];
		color[1] = o.color[1];
		color[2] = o.color[2];
		color[3] = o.color[3];
		
		front = o.front;
		back = o.back;
		left = o.left;
		right = o.right;
		
		size = 2;
		data = new float[size];
		data[0] = 0;//type of powerup
		data[1] = 0;//additional info
		
	}
	virtual SGameMsg* Action(unsigned long t)
	{

		if (iTimer > t)
			iTimer-=t;
		else
			iTimer = 0;
		if (iTimer == 0)
			{
				iActive =1;
				//data[0] = random()%6;
			}
		
		return NULL;
	}
};


const int PBTPD_ENGINE  = 3;
const int PBTPD_ARMOR   = 4;
const int PBTPD_GUN     = 5;

struct STankPrototype : SGameObject
{
	STankPrototype(){}
	STankPrototype(SGameObject o)
	{
		type = o.type;
		iGraphicObjectId = o.iGraphicObjectId;
		iTexture1Id = o.iTexture1Id;
		iTexture2Id = o.iTexture2Id;
		pos = o.pos;
		rot = o.rot;
		color[0] = o.color[0];
		color[1] = o.color[1];
		color[2] = o.color[2];
		color[3] = o.color[3];
		
		front = o.front;
		back = o.back;
		left = o.left;
		right = o.right;
		
		size = 11;
		data = new float[size];
		data[2]=0; // this is the inertator
                data[3]=1; // Engine
		data[4]=1; // Armor
		data[5]=1; // Gun
		data[6]=0; // Frags
		data[7]=0; // X coord of a shell explotion
		data[8]=0; // Y coord of a shell explotion
		data[9]=0; // Powerup Type
		data[10]=0;// Remaining Powerup time
		iActive = o.iActive;
	}
	 virtual SGameMsg* Action(unsigned long t,int i);
};



class IGameInterface
{
	protected:
	vector<SGameMsg> MsgArray;
	public:
	int fr;
	int GameState;
	virtual void ShutDown() = 0;
	virtual vector<SGameObject*> GetGameObjects () = 0;
	virtual vector<int> GetGameEffectObjects () = 0;
	virtual int AddGameObject( SGameObject* obj ) = 0;
	virtual void AddMsg (SGameMsg msg) = 0;
	virtual unsigned long GetTime() = 0;
	virtual void SetTimeOffset(unsigned long o) = 0;
};

class IGraphicInterface
{


	public:

		float LX,LZ;
		int shmd;
        int wf;
        int fc;
	    virtual void SetPos( Vector3f pos ) = 0;
		virtual void AddToPos ( Vector3f v ) = 0;
		virtual void SetAngles (float a, float b) = 0;
		virtual void AddToAngles (float a,float b) = 0;
		virtual Vector3f GetDirection () = 0;
};


#endif
